import sqlite3
import os

# Function that prints out a table
# using SQL select command
def print_table(sql):
 db.execute(sql) 
 all_rows = db.fetchall() 
 for row in all_rows: 
  for i in row: 
   print(i, end=' | \t')
  print() 
 print()


# Remove database if it exists
if os.path.isfile('database.db'):
 os.remove('database.db')

# Create database and open database	
conn = sqlite3.connect('database.db')
db = conn.cursor()

db.execute("PRAGMA foreign_keys = ON")

# create tables

db.execute("""CREATE TABLE  authors
                  (author_ID INTEGER PRIMARY KEY,
                   firstname TEXT,
                   surname TEXT,
                   agent TEXT)""")

db.execute("""CREATE TABLE books
                  (book_ID INTEGER,
                   author_ID INTEGER, 
                   title TEXT,
                   year INTEGER,
                   publisher TEXT,
                   FOREIGN KEY (author_ID) REFERENCES authors(author_ID)
                   on DELETE CASCADE)""")

db.execute("""INSERT INTO authors VALUES(NULL,'JK', Rowling','Neil Blair')""")
db.execute("""INSERT INTO authors VALUES(NULL,'Michael', Morpurgo','David Higham Associates')""")
db.execute("""INSERT INTO authors VALUES(NULL,'Roald', Dahl','David Higham Associates')""")

db.execute("""INSERT INTO books VALUES(1,1,'Harry Potter and the Order of the Phoenix',2003,'Bloomsbury')""")
db.execute("""INSERT INTO books VALUES(2,1,'Fantastic Beasts and where to find them',2001,'Bloomsbury')""")
db.execute("""INSERT INTO books VALUES(3,2,'War Horse',1982,'HarperCollins')""")
db.execute("""INSERT INTO books VALUES(4,2,'Private Peacecful',2003,'HarperCollins')""")
db.execute("""INSERT INTO books VALUES(5,3,'Going Solo',1986,'Penguin')""")
db.execute("""INSERT INTO books VALUES(6,3,'The BFG',1982,'Penguin')""")

print_table("""SELECT * FROM authors""")
print_table("""SELECT * FROM books""")

# What is the difference between the following select statements?
print_table("""SELECT authors.name, books.title from authors, books
        WHERE authors.author_ID=books.author_ID""")

print_table("""SELECT authors.name, books.title from authors, books
        WHERE authors.author_ID=books.author_ID ORDER BY authors.name DESC""")

print_table("""SELECT authors.name, books.title from books INNER JOIN authors ON 
        authors.author_ID=books.author_ID""")

# How do we find out which author wrote a particular book?

# We could execute two SQl statements
db.execute("SELECT author_ID FROM books WHERE title='War Horse'") 
db.execute("SELECT name FROM authors WHERE author_ID=?",db.fetchone())
print(db.fetchone()[0])

# or use a subquery which would be much neater.
db.execute("SELECT name FROM authors WHERE author_ID=(SELECT author_ID FROM books WHERE title='War Horse')") 
print(db.fetchone()[0])

# Now lets try to insert an author that does not exist into the books table
# What happens?
# db.execute("""INSERT INTO books (book_ID,author_ID,title) VALUES(7,4,'Fantastic Mr Fox')""")

#deleting record from authors what happens in books?
sql="DELETE FROM authors WHERE name='Roald Dahl'"
db.execute(sql)
print_table("""SELECT * FROM books""")


# Now do all the above without the
# primary key and foreign key constraint.

conn.commit()
conn.close()
